/** Enumerates over an array, returning both the value and the index. */
export function* enumerate(iterable) {
    let i = 0;
    for (const item of iterable) {
        yield [i, item];
        i++;
    }
}
/** Filters an object by a predicate. */
export function filterObject(obj, predicate) {
    return Object.fromEntries(Object.entries(obj).filter(([key, value]) => predicate(key, value)));
}
/** Maps an object. */
export function mapObject(obj, map) {
    return Object.fromEntries(Object.entries(obj).map(([key, value]) => [key, map(key, value)]));
}
/** Filters and maps an object at the same time. */
export function filterMapObject(obj, predicate) {
    return Object.fromEntries(Object.entries(obj).reduce((acc, [key, value]) => {
        const result = predicate(key, value);
        if (result === null)
            return acc;
        acc.push([key, result]);
        return acc;
    }, []));
}
/** Filter and map an array at the same time. */
export function filterMap(array, predicate) {
    return array.reduce((acc, item) => {
        const result = predicate(item);
        if (result === null)
            return acc;
        acc.push(result);
        return acc;
    }, []);
}
//# sourceMappingURL=iter.js.map