/** @type  */
type TimeFormat<Abbr extends string, Next extends string> = `${`${number}${Abbr}` | `${number}${Abbr} ` | ""}${Next}`;
type Time = TimeFormat<"y", TimeMonth>;
type TimeMonth = TimeFormat<"mo", TimeDay>;
type TimeDay = TimeFormat<"d", TimeHour>;
type TimeHour = TimeFormat<"h", TimeMinute>;
type TimeMinute = TimeFormat<"m", TimeSecond>;
type TimeSecond = TimeFormat<"s", TimeMillisecond>;
type TimeMillisecond = `${number}ms` | "";
export type TimeParseable = Time | number;
/** Convert a string length of time into milliseconds.
 *
 *  The string can be in the format of `1y 2mo 3d 4h 5m 6s 7ms`.
 *  Any of the segments can be omitted.
 *  Whitespace between the segments is optional.
 *  The segments can *not* be in any order.
 *
 *  Examples:
 *  - `ms("6s749ms")` -> `6749`
 *  - `ms("6h")` -> `21600000`
 *  - `ms("8m -14s")` -> `466000`
 */
export declare function ms(time: TimeParseable): number;
/** Formats a duration in milliseconds for `x:xx` or `x:xx:xx` format,
 *  depending on if hours need to be specified.
 */
export declare function formatDuration(duration: TimeParseable): string;
/** Formats a date relative to the current time. */
export declare function formatRelativeTime(date: Date, config?: {
    lowercase?: boolean;
    now?: Date;
}): string;
export {};
